@@ -1,18 +1,24 @@
-from django import forms
 from django.contrib import admin
-from django.core.exceptions import ValidationError
+from django.db.models import Q
 from django.db.models.functions import Lower
 from django.http import HttpResponseRedirect
 from django.shortcuts import render
 from django.utils.safestring import mark_safe
-from django.utils.translation import gettext_lazy as _
-
-from allianceauth.eveonline.models import EveAllianceInfo
 from eveuniverse.models import EveGroup, EveType
 
-from .core.killmails import Killmail
-from .models import Webhook, Tracker
+from allianceauth.eveonline.models import EveAllianceInfo
+
 from . import tasks
+from .constants import (
+    EVE_CATEGORY_ID_FIGHTER,
+    EVE_CATEGORY_ID_SHIP,
+    EVE_CATEGORY_ID_STRUCTURE,
+    EVE_GROUP_MINING_DRONE,
+    EVE_GROUP_ORBITAL_INFRASTRUCTURE,
+)
+from .core.killmails import Killmail
+from .forms import TrackerAdminForm, TrackerAdminKillmailIdForm, field_nice_display
+from .models import Tracker, Webhook
 
 
 @admin.register(Webhook)
@@ -22,158 +28,34 @@ class WebhookAdmin(admin.ModelAdmin):
     ordering = ("name",)
 
     def _messages_in_queue(self, obj):
-        return obj.queue_size()
+        return obj.main_queue.size()
 
     actions = ["send_test_message", "purge_messages"]
 
+    @admin.display(description="Purge queued messages of selected webhooks")
     def purge_messages(self, request, queryset):
         actions_count = 0
         killmails_deleted = 0
         for webhook in queryset:
-            killmails_deleted += webhook.clear_queue()
+            killmails_deleted += webhook.main_queue.clear()
             actions_count += 1
-
         self.message_user(
             request,
             f"Purged queued messages for {actions_count} webhooks, "
             f"deleting a total of {killmails_deleted} messages.",
         )
 
-    purge_messages.short_description = "Purge queued messages of selected webhooks"
-
+    @admin.display(description="Send test message to selected webhooks")
     def send_test_message(self, request, queryset):
         actions_count = 0
         for webhook in queryset:
-            tasks.send_test_message_to_webhook.delay(webhook.pk, request.user.pk)
+            tasks.send_test_message_to_webhook.delay(webhook.pk)
             actions_count += 1
-
         self.message_user(
             request,
-            f"Initiated sending of {actions_count} test messages to "
-            f"selected webhooks. You will receive a notification with the result.",
+            f"Initiated sending of {actions_count} test messages to selected webhooks.",
         )
 
-    send_test_message.short_description = "Send test message to selected webhooks"
-
-
-class TrackerAdminKillmailId(forms.Form):
-    killmail_id = forms.IntegerField()
-
-
-def field_nice_display(name: str) -> str:
-    return name.replace("_", " ").capitalize()
-
-
-class TrackerAdminForm(forms.ModelForm):
-    def clean(self):
-        cleaned_data = super().clean()
-
-        if (
-            cleaned_data["require_max_jumps"]
-            and not cleaned_data["origin_solar_system"]
-        ):
-            raise ValidationError(
-                {
-                    "origin_solar_system": _(
-                        "'Require max jumps' needs an "
-                        f"{field_nice_display('origin_solar_system')} to work"
-                    )
-                }
-            )
-
-        if (
-            cleaned_data["require_max_distance"]
-            and not cleaned_data["origin_solar_system"]
-        ):
-            raise ValidationError(
-                {
-                    "origin_solar_system": _(
-                        "'Require max distance' needs an "
-                        f"{field_nice_display('origin_solar_system')} to work"
-                    )
-                }
-            )
-
-        self._validate_not_same_options_chosen(
-            cleaned_data,
-            "exclude_attacker_alliances",
-            "require_attacker_alliances",
-        )
-        self._validate_not_same_options_chosen(
-            cleaned_data,
-            "exclude_attacker_corporations",
-            "require_attacker_corporations",
-        )
-
-        if (
-            cleaned_data["require_min_attackers"]
-            and cleaned_data["require_max_attackers"]
-            and cleaned_data["require_min_attackers"]
-            > cleaned_data["require_max_attackers"]
-        ):
-            raise ValidationError(
-                {
-                    "require_min_attackers": _(
-                        "Can not be larger than "
-                        f"{field_nice_display('require_max_attackers')}"
-                    )
-                }
-            )
-
-        if (
-            cleaned_data["exclude_high_sec"]
-            and cleaned_data["exclude_low_sec"]
-            and cleaned_data["exclude_null_sec"]
-            and cleaned_data["exclude_w_space"]
-        ):
-            text = ", ".join(
-                [
-                    field_nice_display(x)
-                    for x in [
-                        "exclude_low_sec",
-                        "exclude_null_sec",
-                        "exclude_w_space",
-                        "exclude_high_sec",
-                    ]
-                ]
-            )
-            raise ValidationError(
-                f"Setting all four clauses together does not make sense: {text}"
-            )
-
-        if cleaned_data["exclude_npc_kills"] and cleaned_data["require_npc_kills"]:
-            text = ", ".join(
-                [
-                    field_nice_display(x)
-                    for x in [
-                        "exclude_npc_kills",
-                        "require_npc_kills",
-                    ]
-                ]
-            )
-            raise ValidationError(
-                f"Setting both clauses together does not make sense: {text}"
-            )
-
-    @staticmethod
-    def _validate_not_same_options_chosen(
-        cleaned_data, field_name_1, field_name_2, display_func=lambda x: x
-    ) -> None:
-        same_options = set(cleaned_data[field_name_1]).intersection(
-            set(cleaned_data[field_name_2])
-        )
-        if same_options:
-            same_options_text = ", ".join(
-                map(
-                    str,
-                    [display_func(x) for x in same_options],
-                )
-            )
-            raise ValidationError(
-                f"Can not choose same options for {field_nice_display(field_name_1)} "
-                f"& {field_nice_display(field_name_2)}: {same_options_text}"
-            )
-
 
 @admin.register(Tracker)
 class TrackerAdmin(admin.ModelAdmin):
@@ -184,6 +66,8 @@ class TrackerAdmin(admin.ModelAdmin):
         "webhook",
         "identify_fleets",
         "_clauses",
+        "_pings",
+        "_color",
     )
     list_filter = (
         "is_enabled",
@@ -192,6 +76,41 @@ class TrackerAdmin(admin.ModelAdmin):
     )
     ordering = ("name",)
 
+    def get_queryset(self, request):
+        qs = super().get_queryset(request)
+        return qs.prefetch_related(
+            "exclude_attacker_alliances",
+            "require_attacker_alliances",
+            "exclude_attacker_corporations",
+            "require_attacker_corporations",
+            "exclude_attacker_states",
+            "require_attacker_states",
+            "require_victim_states",
+            "require_victim_alliances",
+            "require_victim_corporations",
+            "require_regions",
+            "require_constellations",
+            "require_solar_systems",
+            "require_attackers_ship_groups",
+            "require_attackers_ship_types",
+            "require_victim_ship_groups",
+            "require_victim_ship_types",
+            "ping_groups",
+        )
+
+    def _color(self, obj):
+        html = (
+            f'<input type="color" value="{obj.color}" disabled>' if obj.color else "-"
+        )
+        return mark_safe(html)
+
+    def _pings(self, obj):
+        parts = [f"@{group.name}" for group in obj.ping_groups.all()]
+        if obj.ping_type != Tracker.ChannelPingType.NONE:
+            parts.append(obj.get_ping_type_display())
+
+        return sorted(parts, key=str.casefold) if parts else None
+
     def _clauses(self, obj):
         clauses = list()
         for field, func in [
@@ -204,6 +123,9 @@ class TrackerAdmin(admin.ModelAdmin):
             ("require_attacker_corporations", self._add_to_clauses_2),
             ("require_victim_alliances", self._add_to_clauses_2),
             ("require_victim_corporations", self._add_to_clauses_2),
+            ("exclude_attacker_states", self._add_to_clauses_2),
+            ("require_attacker_states", self._add_to_clauses_2),
+            ("require_victim_states", self._add_to_clauses_2),
             ("exclude_blue_attackers", self._add_to_clauses_1),
             ("require_blue_victim", self._add_to_clauses_1),
             ("require_min_attackers", self._add_to_clauses_1),
@@ -219,6 +141,7 @@ class TrackerAdmin(admin.ModelAdmin):
             ("require_attackers_ship_groups", self._add_to_clauses_2),
             ("require_attackers_ship_types", self._add_to_clauses_2),
             ("require_victim_ship_groups", self._add_to_clauses_2),
+            ("require_victim_ship_types", self._add_to_clauses_2),
             ("exclude_npc_kills", self._add_to_clauses_1),
             ("require_npc_kills", self._add_to_clauses_1),
         ]:
@@ -232,17 +155,32 @@ class TrackerAdmin(admin.ModelAdmin):
 
     def _add_to_clauses_2(self, clauses, obj, field):
         if getattr(obj, field).count() > 0:
-            text = ", ".join(map(str, getattr(obj, field).all()))
+            text = ", ".join(sorted(map(str, getattr(obj, field).all())))
             self._append_field_to_clauses(clauses, field, text)
 
     def _append_field_to_clauses(self, clauses, field, text):
         clauses.append(f"{field_nice_display(field)} = {text}")
 
-    actions = ["run_test_killmail"]
+    actions = ["disable_tracker", "enable_tracker", "reset_color", "run_test_killmail"]
 
+    @admin.display(description="Reset color for selected trackers")
+    def reset_color(self, request, queryset):
+        queryset.update(color="")
+
+    @admin.display(description="Enable selected trackers")
+    def enable_tracker(self, request, queryset):
+        queryset.update(is_enabled=True)
+        self.message_user(request, f"{queryset.count()} trackers enabled.")
+
+    @admin.display(description="Disable selected trackers")
+    def disable_tracker(self, request, queryset):
+        queryset.update(is_enabled=False)
+        self.message_user(request, f"{queryset.count()} trackers disabled.")
+
+    @admin.display(description="Run test killmail with selected trackers")
     def run_test_killmail(self, request, queryset):
         if "apply" in request.POST:
-            form = TrackerAdminKillmailId(request.POST)
+            form = TrackerAdminKillmailIdForm(request.POST)
             if form.is_valid():
                 killmail_id = form.cleaned_data["killmail_id"]
                 killmail = Killmail.create_from_zkb_api(killmail_id)
@@ -256,7 +194,6 @@ class TrackerAdmin(admin.ModelAdmin):
                             ignore_max_age=True,
                         )
                         actions_count += 1
-
                     self.message_user(
                         request,
                         (
@@ -269,16 +206,14 @@ class TrackerAdmin(admin.ModelAdmin):
                         request,
                         "Failed to load killmail with ID {killmail_id} from ZKB",
                     )
-
             return HttpResponseRedirect(request.get_full_path())
-        else:
-            last_killmail_id = request.session.get("last_killmail_id")
-            if last_killmail_id:
-                initial = {"killmail_id": last_killmail_id}
-            else:
-                initial = None
-            form = TrackerAdminKillmailId(initial=initial)
 
+        last_killmail_id = request.session.get("last_killmail_id")
+        if last_killmail_id:
+            initial = {"killmail_id": last_killmail_id}
+        else:
+            initial = None
+        form = TrackerAdminKillmailIdForm(initial=initial)
         return render(
             request,
             "admin/killtracker_killmail_id.html",
@@ -289,8 +224,6 @@ class TrackerAdmin(admin.ModelAdmin):
             },
         )
 
-    run_test_killmail.short_description = "Run test killmail with selected trackers"
-
     autocomplete_fields = ["origin_solar_system"]
 
     filter_horizontal = (
@@ -300,22 +233,28 @@ class TrackerAdmin(admin.ModelAdmin):
         "require_attacker_corporations",
         "require_victim_alliances",
         "require_victim_corporations",
+        "exclude_attacker_states",
+        "require_attacker_states",
+        "require_victim_states",
         "require_regions",
         "require_constellations",
         "require_solar_systems",
         "require_attackers_ship_groups",
         "require_attackers_ship_types",
         "require_victim_ship_groups",
+        "require_victim_ship_types",
+        "ping_groups",
     )
 
     fieldsets = (
-        (None, {"fields": ("name", "description", "is_enabled")}),
+        (None, {"fields": ("name", "description", "is_enabled", "color")}),
         (
             "Discord Configuration",
             {
                 "fields": (
                     "webhook",
                     "ping_type",
+                    "ping_groups",
                     "is_posting_name",
                 ),
             },
@@ -352,6 +291,16 @@ class TrackerAdmin(admin.ModelAdmin):
                 ),
             },
         ),
+        (
+            "Auth State",
+            {
+                "fields": (
+                    "exclude_attacker_states",
+                    "require_attacker_states",
+                    "require_victim_states",
+                ),
+            },
+        ),
         (
             "Fleet detection",
             {
@@ -379,6 +328,7 @@ class TrackerAdmin(admin.ModelAdmin):
                     "require_attackers_ship_groups",
                     "require_attackers_ship_types",
                     "require_victim_ship_groups",
+                    "require_victim_ship_types",
                 ),
             },
         ),
@@ -399,9 +349,61 @@ class TrackerAdmin(admin.ModelAdmin):
                 Lower("alliance_name")
             )
         elif db_field.name == "require_attackers_ship_groups":
-            kwargs["queryset"] = EveGroup.objects.all().order_by(Lower("name"))
-
+            kwargs["queryset"] = EveGroup.objects.filter(
+                eve_category_id__in=[
+                    EVE_CATEGORY_ID_STRUCTURE,
+                    EVE_CATEGORY_ID_SHIP,
+                    EVE_CATEGORY_ID_FIGHTER,
+                ],
+                published=True,
+            ).order_by(Lower("name"))
         elif db_field.name == "require_attackers_ship_types":
-            kwargs["queryset"] = EveType.objects.all().order_by(Lower("name"))
+            kwargs["queryset"] = (
+                EveType.objects.select_related("eve_group")
+                .filter(
+                    eve_group__eve_category_id__in=[
+                        EVE_CATEGORY_ID_STRUCTURE,
+                        EVE_CATEGORY_ID_SHIP,
+                        EVE_CATEGORY_ID_FIGHTER,
+                    ],
+                    published=True,
+                )
+                .order_by(Lower("name"))
+            )
+        elif db_field.name == "require_victim_ship_groups":
+            kwargs["queryset"] = EveGroup.objects.filter(
+                (
+                    Q(
+                        eve_category_id__in=[
+                            EVE_CATEGORY_ID_STRUCTURE,
+                            EVE_CATEGORY_ID_SHIP,
+                            EVE_CATEGORY_ID_FIGHTER,
+                        ]
+                    )
+                    & Q(published=True)
+                )
+                | (Q(id=EVE_GROUP_MINING_DRONE) & Q(published=True))
+                | Q(id=EVE_GROUP_ORBITAL_INFRASTRUCTURE)
+            ).order_by(Lower("name"))
+
+        elif db_field.name == "require_victim_ship_types":
+            kwargs["queryset"] = (
+                EveType.objects.select_related("eve_group")
+                .filter(
+                    (
+                        Q(
+                            eve_group__eve_category_id__in=[
+                                EVE_CATEGORY_ID_STRUCTURE,
+                                EVE_CATEGORY_ID_SHIP,
+                                EVE_CATEGORY_ID_FIGHTER,
+                            ]
+                        )
+                        & Q(published=True)
+                    )
+                    | (Q(eve_group_id=EVE_GROUP_MINING_DRONE) & Q(published=True))
+                    | Q(eve_group_id=EVE_GROUP_ORBITAL_INFRASTRUCTURE)
+                )
+                .order_by(Lower("name"))
+            )
 
         return super().formfield_for_manytomany(db_field, request, **kwargs)
