@@ -1,24 +1,22 @@
-from datetime import datetime
-from dataclasses import dataclass, asdict
 import json
+from dataclasses import asdict, dataclass
+from datetime import datetime
 from typing import List, Optional, Set
 
-from dacite import from_dict, DaciteError
 import requests
+from dacite import DaciteError, from_dict
 
 from django.core.cache import cache
 from django.utils.dateparse import parse_datetime
 
 from allianceauth.services.hooks import get_extension_logger
-
 from app_utils.json import JSONDateTimeDecoder, JSONDateTimeEncoder
 from app_utils.logging import LoggerAddTag
 
-from .. import __title__, USER_AGENT_TEXT
-from ..app_settings import KILLTRACKER_REDISQ_TTW, KILLTRACKER_REDISQ_LOCK_TIMEOUT
+from .. import USER_AGENT_TEXT, __title__
+from ..app_settings import KILLTRACKER_REDISQ_LOCK_TIMEOUT, KILLTRACKER_REDISQ_TTW
 from ..providers import esi
 
-
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
 ZKB_REDISQ_URL = "https://redisq.zkillboard.com/listen.php"
@@ -131,28 +129,24 @@ class Killmail(_KillmailBase):
     def __repr__(self):
         return f"{type(self).__name__}(id={self.id})"
 
-    def attackers_alliance_ids(self) -> List[int]:
-        """returns alliance IDs of all attackers"""
-        return [
-            obj.alliance_id for obj in self.attackers if obj.alliance_id is not None
-        ]
+    def attackers_distinct_alliance_ids(self) -> Set[int]:
+        """Return distinct alliance IDs of all attackers."""
+        return {obj.alliance_id for obj in self.attackers if obj.alliance_id}
 
-    def attackers_corporation_ids(self) -> List[int]:
-        """returns corporation IDs of all attackers"""
-        return [
-            obj.corporation_id
-            for obj in self.attackers
-            if obj.corporation_id is not None
-        ]
+    def attackers_distinct_corporation_ids(self) -> Set[int]:
+        """Return distinct corporation IDs of all attackers."""
+        return {obj.corporation_id for obj in self.attackers if obj.corporation_id}
+
+    def attackers_distinct_character_ids(self) -> Set[int]:
+        """Return distinct character IDs of all attackers."""
+        return {obj.character_id for obj in self.attackers if obj.character_id}
 
     def attackers_ship_type_ids(self) -> List[int]:
-        """returns ship type IDs of all attackers as list (including duplicates!)"""
-        return [
-            obj.ship_type_id for obj in self.attackers if obj.ship_type_id is not None
-        ]
+        """Returns ship type IDs of all attackers with duplicates."""
+        return [obj.ship_type_id for obj in self.attackers if obj.ship_type_id]
 
     def entity_ids(self) -> Set[int]:
-        """returns set of IDs of all entities that are not None"""
+        """Return distinct IDs of all entities (excluding None)."""
         ids = {
             self.victim.character_id,
             self.victim.corporation_id,
@@ -175,8 +169,8 @@ class Killmail(_KillmailBase):
         ids.discard(None)
         return ids
 
-    def ship_type_ids(self) -> Set[int]:
-        """returns ship type IDs of all entities that are not None as set"""
+    def ship_type_distinct_ids(self) -> Set[int]:
+        """Return distinct ship type IDs of all entities that are not None."""
         ids = set(self.attackers_ship_type_ids())
         ids.add(self.victim.ship_type_id)
         return ids
@@ -247,52 +241,52 @@ class Killmail(_KillmailBase):
         killmail_json = cache.get(cache_key)
         if killmail_json:
             return Killmail.from_json(killmail_json)
-        else:
-            logger.info(
-                "Trying to fetch killmail from ZKB API with killmail ID %d ...",
-                killmail_id,
+
+        logger.info(
+            "Trying to fetch killmail from ZKB API with killmail ID %d ...",
+            killmail_id,
+        )
+        url = f"{ZKB_API_URL}killID/{killmail_id}/"
+        r = requests.get(
+            url, timeout=REQUESTS_TIMEOUT, headers={"User-Agent": USER_AGENT_TEXT}
+        )
+        r.raise_for_status()
+        zkb_data = r.json()
+        if not zkb_data:
+            logger.warning(
+                "ZKB API did not return any data for killmail ID %d", killmail_id
             )
-            url = f"{ZKB_API_URL}killID/{killmail_id}/"
-            r = requests.get(
-                url, timeout=REQUESTS_TIMEOUT, headers={"User-Agent": USER_AGENT_TEXT}
+            return None
+
+        logger.debug("data:\n%s", zkb_data)
+        try:
+            killmail_zkb = zkb_data[0]
+        except KeyError:
+            return None
+
+        killmail_esi = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
+            killmail_id=killmail_id, killmail_hash=killmail_zkb["zkb"]["hash"]
+        ).results()
+        if not killmail_esi:
+            logger.warning(
+                "ESI did not return any data for killmail ID %d", killmail_id
             )
-            r.raise_for_status()
-            zkb_data = r.json()
-            if not zkb_data:
-                logger.warning(
-                    "ZKB API did not return any data for killmail ID %d", killmail_id
-                )
-                return None
+            return None
 
-            logger.debug("data:\n%s", zkb_data)
-            try:
-                killmail_zkb = zkb_data[0]
-            except KeyError:
-                return None
+        # esi returns datetime, but _create_from_dict() expects a string in
+        # same format as returned from zkb redisq
+        killmail_esi["killmail_time"] = killmail_esi["killmail_time"].strftime(
+            "%Y-%m-%dT%H:%M:%SZ"
+        )
 
-            killmail_esi = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
-                killmail_id=killmail_id, killmail_hash=killmail_zkb["zkb"]["hash"]
-            ).results()
-            if not killmail_esi:
-                logger.warning(
-                    "ESI did not return any data for killmail ID %d", killmail_id
-                )
-                return None
-
-            # esi returns datetime, but _create_from_dict() expects a string in
-            # same format as returned from zkb redisq
-            killmail_esi["killmail_time"] = killmail_esi["killmail_time"].strftime(
-                "%Y-%m-%dT%H:%M:%SZ"
-            )
-
-            killmail_dict = {
-                "killID": killmail_id,
-                "killmail": killmail_esi,
-                "zkb": killmail_zkb["zkb"],
-            }
-            killmail = cls._create_from_dict(killmail_dict)
-            cache.set(key=cache_key, value=killmail.asjson())
-            return killmail
+        killmail_dict = {
+            "killID": killmail_id,
+            "killmail": killmail_esi,
+            "zkb": killmail_zkb["zkb"],
+        }
+        killmail = cls._create_from_dict(killmail_dict)
+        cache.set(key=cache_key, value=killmail.asjson())
+        return killmail
 
     @staticmethod
     def _create_from_dict(package_data: dict) -> "Killmail":
