@@ -1,50 +1,50 @@
+import json
 from copy import deepcopy
 from datetime import timedelta
-import json
-from typing import Optional, List
+from typing import List, Optional
 from urllib.parse import urljoin
 
 import dhooks_lite
 from requests.exceptions import HTTPError
 from simple_mq import SimpleMQ
 
-from django.core.cache import cache
-from django.contrib.auth.models import Group
+from django.contrib.auth.models import Group, User
 from django.contrib.staticfiles.storage import staticfiles_storage
+from django.core.cache import cache
 from django.db import models
-from django.utils.translation import gettext_lazy as _
 from django.utils.timezone import now
-
-from allianceauth.eveonline.evelinks import eveimageserver, zkillboard, dotlan
-from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
-from allianceauth.services.hooks import get_extension_logger
-from allianceauth.services.modules.discord.models import DiscordUser
-
-from app_utils.django import app_labels
-from app_utils.logging import LoggerAddTag
-from app_utils.json import JSONDateTimeEncoder, JSONDateTimeDecoder
-from app_utils.urls import site_absolute_url
-from app_utils.views import humanize_value
-from eveuniverse.helpers import meters_to_ly, EveEntityNameResolver
+from django.utils.translation import gettext_lazy as _
+from eveuniverse.helpers import EveEntityNameResolver, meters_to_ly
 from eveuniverse.models import (
     EveConstellation,
+    EveEntity,
+    EveGroup,
     EveRegion,
     EveSolarSystem,
-    EveGroup,
-    EveEntity,
     EveType,
 )
 
-from . import __title__, APP_NAME, HOMEPAGE_URL, __version__
+from allianceauth.authentication.models import State
+from allianceauth.eveonline.evelinks import dotlan, eveimageserver, zkillboard
+from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
+from allianceauth.services.hooks import get_extension_logger
+from allianceauth.services.modules.discord.models import DiscordUser
+from app_utils.allianceauth import get_redis_client
+from app_utils.django import app_labels
+from app_utils.json import JSONDateTimeDecoder, JSONDateTimeEncoder
+from app_utils.logging import LoggerAddTag
+from app_utils.urls import site_absolute_url
+from app_utils.views import humanize_value
+
+from . import APP_NAME, HOMEPAGE_URL, __title__, __version__
 from .app_settings import (
     KILLTRACKER_KILLMAIL_MAX_AGE_FOR_TRACKER,
     KILLTRACKER_WEBHOOK_SET_AVATAR,
 )
-from .core.killmails import EntityCount, Killmail, TrackerInfo, ZKB_KILLMAIL_BASEURL
+from .core.killmails import ZKB_KILLMAIL_BASEURL, EntityCount, Killmail, TrackerInfo
 from .exceptions import WebhookTooManyRequests
 from .managers import EveKillmailManager, TrackerManager, WebhookManager
 
-
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
 
@@ -278,10 +278,9 @@ class Webhook(models.Model):
             self.error_queue = self._create_queue("error")
 
     def _create_queue(self, suffix: str) -> Optional[SimpleMQ]:
+        redis_client = get_redis_client()
         return (
-            SimpleMQ(
-                cache.get_master_client(), f"{__title__}_webhook_{self.pk}_{suffix}"
-            )
+            SimpleMQ(redis_client, f"{__title__}_webhook_{self.pk}_{suffix}")
             if self.pk
             else None
         )
@@ -418,7 +417,9 @@ class Webhook(models.Model):
     @staticmethod
     def create_message_link(name: str, url: str) -> str:
         """Create link for a Discord message"""
-        return f"[{str(name)}]({str(url)})"
+        if name and url:
+            return f"[{str(name)}]({str(url)})"
+        return str(name)
 
 
 class Tracker(models.Model):
@@ -508,6 +509,26 @@ class Tracker(models.Model):
         blank=True,
         help_text="only include killmails with attackers from one of these corporations",
     )
+    exclude_attacker_states = models.ManyToManyField(
+        State,
+        related_name="+",
+        default=None,
+        blank=True,
+        help_text=(
+            "exclude killmails with characters belonging "
+            "to users with these Auth states"
+        ),
+    )
+    require_attacker_states = models.ManyToManyField(
+        State,
+        related_name="+",
+        default=None,
+        blank=True,
+        help_text=(
+            "only include killmails with characters belonging "
+            "to users with these Auth states"
+        ),
+    )
     require_victim_alliances = models.ManyToManyField(
         EveAllianceInfo,
         related_name="+",
@@ -527,6 +548,16 @@ class Tracker(models.Model):
             "to one of these corporations"
         ),
     )
+    require_victim_states = models.ManyToManyField(
+        State,
+        related_name="+",
+        default=None,
+        blank=True,
+        help_text=(
+            "only include killmails where the victim characters belong "
+            "to users with these Auth states"
+        ),
+    )
     identify_fleets = models.BooleanField(
         default=False,
         help_text="when true: kills are interpreted and shown as fleet kills",
@@ -747,7 +778,9 @@ class Tracker(models.Model):
 
         # Make sure all ship types are in the local database
         if self.has_type_clause:
-            EveType.objects.bulk_get_or_create_esi(ids=killmail.ship_type_ids())
+            EveType.objects.bulk_get_or_create_esi(
+                ids=killmail.ship_type_distinct_ids()
+            )
 
         # apply filters
         is_matching = True
@@ -813,22 +846,22 @@ class Tracker(models.Model):
 
             if is_matching and self.exclude_attacker_alliances.exists():
                 is_matching = self.exclude_attacker_alliances.exclude(
-                    alliance_id__in=killmail.attackers_alliance_ids()
+                    alliance_id__in=killmail.attackers_distinct_alliance_ids()
                 ).exists()
 
             if is_matching and self.require_attacker_alliances.exists():
                 is_matching = self.require_attacker_alliances.filter(
-                    alliance_id__in=killmail.attackers_alliance_ids()
+                    alliance_id__in=killmail.attackers_distinct_alliance_ids()
                 ).exists()
 
             if is_matching and self.exclude_attacker_corporations.exists():
                 is_matching = self.exclude_attacker_corporations.exclude(
-                    corporation_id__in=killmail.attackers_corporation_ids()
+                    corporation_id__in=killmail.attackers_distinct_corporation_ids()
                 ).exists()
 
             if is_matching and self.require_attacker_corporations.exists():
                 is_matching = self.require_attacker_corporations.filter(
-                    corporation_id__in=killmail.attackers_corporation_ids()
+                    corporation_id__in=killmail.attackers_distinct_corporation_ids()
                 ).exists()
 
             if is_matching and self.require_victim_alliances.exists():
@@ -841,6 +874,30 @@ class Tracker(models.Model):
                     corporation_id=killmail.victim.corporation_id
                 ).exists()
 
+            if is_matching and self.require_attacker_states.exists():
+                is_matching = User.objects.filter(
+                    profile__state__in=list(self.require_attacker_states.all()),
+                    character_ownerships__character__character_id__in=(
+                        killmail.attackers_distinct_character_ids()
+                    ),
+                ).exists()
+
+            if is_matching and self.exclude_attacker_states.exists():
+                is_matching = not User.objects.filter(
+                    profile__state__in=list(self.exclude_attacker_states.all()),
+                    character_ownerships__character__character_id__in=(
+                        killmail.attackers_distinct_character_ids()
+                    ),
+                ).exists()
+
+            if is_matching and self.require_victim_states.exists():
+                is_matching = User.objects.filter(
+                    profile__state__in=list(self.require_victim_states.all()),
+                    character_ownerships__character__character_id=(
+                        killmail.victim.character_id
+                    ),
+                ).exists()
+
             if is_matching and self.require_victim_ship_groups.exists():
                 ship_types_matching_qs = EveType.objects.filter(
                     eve_group_id__in=list(
@@ -868,9 +925,8 @@ class Tracker(models.Model):
                     )
 
             if is_matching and self.require_attackers_ship_groups.exists():
-                attackers_ship_type_ids = killmail.attackers_ship_type_ids()
                 ship_types_matching_qs = EveType.objects.filter(
-                    id__in=attackers_ship_type_ids
+                    id__in=set(killmail.attackers_ship_type_ids())
                 ).filter(
                     eve_group_id__in=list(
                         self.require_attackers_ship_groups.values_list("id", flat=True)
@@ -883,9 +939,8 @@ class Tracker(models.Model):
                     )
 
             if is_matching and self.require_attackers_ship_types.exists():
-                attackers_ship_type_ids = killmail.attackers_ship_type_ids()
                 ship_types_matching_qs = EveType.objects.filter(
-                    id__in=attackers_ship_type_ids
+                    id__in=set(killmail.attackers_ship_type_ids())
                 ).filter(
                     id__in=list(
                         self.require_attackers_ship_types.values_list("id", flat=True)
@@ -1016,16 +1071,21 @@ class Tracker(models.Model):
 
         # victim
         if killmail.victim.alliance_id:
-            victim_organization = EveEntity.objects.get(id=killmail.victim.alliance_id)
+            victim_organization = resolver.to_name(killmail.victim.alliance_id)
             victim_org_url = zkillboard.alliance_url(killmail.victim.alliance_id)
-        elif killmail.victim.corporation_id:
-            victim_organization = EveEntity.objects.get(
-                id=killmail.victim.corporation_id
+            victim_org_icon_url = eveimageserver.corporation_logo_url(
+                killmail.victim.alliance_id, size=self.ICON_SIZE
             )
+        elif killmail.victim.corporation_id:
+            victim_organization = resolver.to_name(killmail.victim.corporation_id)
             victim_org_url = zkillboard.corporation_url(killmail.victim.corporation_id)
+            victim_org_icon_url = eveimageserver.corporation_logo_url(
+                killmail.victim.corporation_id, size=self.ICON_SIZE
+            )
         else:
             victim_organization = None
             victim_org_url = None
+            victim_org_icon_url = None
 
         if killmail.victim.corporation_id:
             victim_corporation_zkb_link = self._corporation_zkb_link(
@@ -1036,10 +1096,9 @@ class Tracker(models.Model):
 
         if killmail.victim.character_id:
             victim_character_zkb_link = self._character_zkb_link(
-                killmail.victim.character_id,
-                resolver,
+                killmail.victim.character_id, resolver
             )
-            victim_str = f"{victim_character_zkb_link} ({victim_corporation_zkb_link}) "
+            victim_str = f"{victim_character_zkb_link} ({victim_corporation_zkb_link})"
         elif killmail.victim.corporation_id:
             victim_str = victim_corporation_zkb_link
         else:
@@ -1144,9 +1203,7 @@ class Tracker(models.Model):
                     main_org_icon_url = eveimageserver.alliance_logo_url(
                         main_org.id, size=self.ICON_SIZE
                     )
-
                 main_org_text = f" | Main group: {main_org_link} ({main_org.count})"
-
             else:
                 show_as_fleetkill = False
 
@@ -1201,11 +1258,11 @@ class Tracker(models.Model):
         # TODO This is a workaround for Embed.Author.name. Address in dhooks_lite
         author = (
             dhooks_lite.Author(
-                name=victim_organization.name if victim_organization.name else "?",
+                name=victim_organization if victim_organization else "?",
                 url=victim_org_url,
-                icon_url=victim_organization.icon_url(),
+                icon_url=victim_org_icon_url,
             )
-            if victim_organization and victim_org_url
+            if victim_organization and victim_org_url and victim_org_icon_url
             else None
         )
         zkb_icon_url = urljoin(
